odoo.define('enercoop_cdc.operation_graph', function (require) {
    "use strict";

    var qweb = require('web.qweb');
    var viewRegistry = require('web.view_registry');
    var core = require('web.core');
    var QWeb = core.qweb;

    let chartValues;
    const Renderer = qweb.Renderer.extend({
        events:_.extend({}, qweb.Renderer.prototype.events, {
            'click .o_enercoop_button': '_onChartClicked',
        }),

        _render: async function () {
            var self = this;
            var chart_line_conso;
            var chart_line_prod;
            var chart_donuts_conso;
            var chart_donuts_prod;
            var chart_histo_conso;
            var chart_histo_prod;

            if (chart_line_conso != undefined){
                chart_line_conso.destroy();
            }
                if (chart_line_prod != undefined){
                chart_line_prod.destroy();
            }
            if (chart_donuts_conso != undefined){
                chart_donuts_conso.destroy();
            }
            if (chart_donuts_prod != undefined){
                chart_donuts_prod.destroy();
            }
            if (chart_histo_conso != undefined){
                chart_histo_conso.destroy();
            }
            if (chart_histo_prod != undefined){
                chart_histo_prod.destroy();
            }
            await this._super(...arguments);

            var result = self.state.chartValues;

            var line_conso = self.$el.find('#line_chart_conso');
            var donuts_conso = self.$el.find('#donuts_chart_conso');
            var histo_conso = self.$el.find('#histo_chart_conso');
            var line_prod = self.$el.find('#line_chart_prod');
            var donuts_prod = self.$el.find('#donuts_chart_prod');
            var histo_prod = self.$el.find('#histo_chart_prod');

            if (result.scale != 'week' && result.scale != 'day'){
                line_conso.hide();
                line_prod.hide();
            }

            if (result.scale == 'week' || result.scale == 'day'){
                chart_line_conso = new Chart(
                    line_conso,
                    Object.assign({}, result.line_chart_conso_line)
                );
            }

	    	chart_donuts_conso = new Chart(
	    		donuts_conso,
	    		Object.assign({}, result.donuts_chart_conso)
	    	);

	    	chart_histo_conso = new Chart(
	    		histo_conso,
	    		Object.assign({}, result.histo_chart_conso)
	    	);

            if (result.scale == 'week' || result.scale == 'day'){
                chart_line_prod = new Chart(
                    line_prod,
                    Object.assign({}, result.line_chart_prod_line)
                );
                console.log(chart_line_prod);
            }

	    	chart_donuts_prod = new Chart(
	    		donuts_prod,
	    		Object.assign({}, result.donuts_chart_prod)
	    	);

	    	chart_histo_prod = new Chart(
	    		histo_prod,
	    		Object.assign({}, result.histo_chart_prod)
	    	);
            this.$el.find('.o_enercoop_button').removeClass('active');
            if (this.state.scale) {
                var $but = this.$el.find('.o_enercoop_button[data-mode="' + this.state.scale + '"]')
            }
            else {
                var $but = this.$el.find('.o_enercoop_button[data-mode="month"]')
            }
            $but.addClass('active');

        },

        _onChartClicked: function (e) {
            e.preventDefault();
            var $action = $(e.currentTarget);
            $action.addClass('active');
            var context = JSON.parse($action.attr('context').replace(/'/g, '"'));
            var first_day = context.first_day;
            var last_day = context.last_day;
            this.trigger_up('chart_open_action', {
                action_name: $action.attr('name'),
                action_context: $action.attr('context'),
                action_first_day: first_day,
                action_last_day: last_day,
            });
        },

    });

    const Model = qweb.Model.extend({
        /**
         * @override
         */
        init: function () {
            this.chartValues = {};
            this._super.apply(this, arguments);
        },

        /**
         * @override
         */
        __get: function (localID) {
            var result = this._super.apply(this, arguments);
            if (_.isObject(result)) {
                result.chartValues = this.chartValues[localID];
            }
            return result;
        },

        /**
         * @override
         * @returns {Promise}
         */
        __load: function (params) {
            if ('scales' in params) {
                this.scales = params.scales;
                this.first_day = params.first_day;
                this.last_day = params.last_day;
            }
            return this._loadChart(this._super.apply(this, arguments), params);
        },
        /**
         * @override
         * @returns {Promise}
         */
        __reload: function (handle, params) {
            if ('scales' in params) {
                this.scales = params.scales;
                this.first_day = params.first_day;
                this.last_day = params.last_day;
            }
            return this._loadChart(this._super.apply(this, arguments), params);
        },

        /**
         * @private
         * @param {Promise} super_def a promise that resolves with a dataPoint id
         * @returns {Promise -> string} resolves to the dataPoint id
         */
        _loadChart: function (super_def, params) {
            var self = this;

            var enercoop_counter_id = self._state.context.search_default_enercoop_delivery_operation_ids
            var domain = self._state.domain
            var chart_def = this._rpc({
                model: 'enercoop.operation',
                method: 'graph_view',
                args: [domain, this.scales, this.first_day, this.last_day, enercoop_counter_id],
                context: this._state.context,
            });
            return Promise.all([super_def, chart_def]).then(function(results) {
                var id = results[0];
                chartValues = results[1];
                self.chartValues[id] = chartValues;
                return id;
            });

        },

        setScale: function (scale) {
            this.scales = scale;
        },

        setDate: function (first_day, last_day) {
            this.first_day = first_day;
            this.last_day = last_day;
        },

    });

    const Controller = qweb.Controller.extend({
        custom_events: _.extend({}, qweb.Controller.prototype.custom_events, {
            chart_open_action: '_onChartClicked',
            cal_open_action: '_onCalClicked',
        }),

        init: function (parent, model, renderer, params) {
            this._super.apply(this, arguments);
            this.scales = params.scales;
            this.first_day = params.first_day;
            this.last_day = params.last_day;
        },

        /**
         * @private
         * @param {OdooEvent} e
         */
        _onChartClicked: function (e) {
            this.model.setScale(e.data.action_name);
            this.model.setDate(e.data.action_first_day, e.data.action_last_day);
            var state = this.model.get();
            state.scale = e.data.action_name
            var context = e.data.action_context
            state.first_day = e.data.action_first_day
            state.last_day = e.data.action_last_day
            this.reload();
        },

        /**
         * @private
         * @param {OdooEvent} e
         */
        _onCalClicked: function (e) {
            state.date_start = e.data.date_start
            state.date_end = e.data.date_end
            this.reload();
        },
    });

    var OperationGraph = qweb.View.extend({
        withSearchBar: true,
        searchMenuTypes: ['filter', 'favorite'],
        jsLibs: [
            '/web/static/lib/Chart/Chart.js',
            '/enercoop_cdc/static/src/lib/chartjs-plugin-datalabels/chartjs-plugin-datalabels.min.js',
            '/web/static/lib/moment/moment.js',
        ],
        config: _.extend({}, qweb.View.prototype.config, {
            Model: Model,
            Renderer: Renderer,
            Controller: Controller,
        }),

        init: function (viewInfo, params) {
            this._super.apply(this, arguments);
            this.controllerParams.scales = 'month';
            this.loadParams.scales = 'month';
            this.controllerParams.first_day = '';
            this.controllerParams.last_day = '';
            this.loadParams.first_day = '';
            this.loadParams.last_day = '';
        },

    });

    viewRegistry.add('operation_graph', OperationGraph);
});
